#include "AudioIO.hxx"
#include "AudioManager.hxx"
#include "AudioIn.hxx"
#include "AudioOut.hxx"
#include "MyCLAMAudioIOApp.hxx"

using namespace CLAM;

MyCLAMAudioIOApp::MyCLAMAudioIOApp()
	: mIsRunning(false)
{
	mThread.SetThreadCode(makeMemberFunctor0((*this), MyCLAMAudioIOApp, thread_code));
}

MyCLAMAudioIOApp::~MyCLAMAudioIOApp()
{
	Stop();
}

void MyCLAMAudioIOApp::Start()
{
	if(mIsRunning) Stop();
	mIsRunning = true;
	mThread.Start();
}

void MyCLAMAudioIOApp::Stop()
{
	if(mIsRunning)
	{
		mIsRunning = false;
		mThread.Stop();
	}
}

void MyCLAMAudioIOApp::thread_code()
{
	TData sampleRate = TData(22050.0); // sample rate -> 44100 Hz
	TSize frameSize = 512;             // frame size
	AudioManager manager((int)sampleRate,(int)frameSize);

	AudioIn channelIn;   // in
	AudioOut channelOut; // out

	AudioIOConfig audioInCfg;  // in cfg
	AudioIOConfig audioOutCfg; // out cfg

	audioInCfg.SetChannelID(0);        // int channel id=0
	channelIn.Configure(audioInCfg);   // configure in

	audioOutCfg.SetChannelID(0);       // out channel id=0
	channelOut.Configure(audioOutCfg); // configure out
    
	manager.Start();    // start manager
	channelIn.Start();  // start in
	channelOut.Start(); // start out

	Audio audioFrame;
	audioFrame.SetSize(frameSize);

	while(mIsRunning)
	{
		channelIn.Do(audioFrame);  // rec
		channelOut.Do(audioFrame); // play
	}

	channelIn.Stop();
	channelOut.Stop();

}

// END

